/*
 * Scilab ( http://www.scilab.org/ ) - This file is part of Scilab
 * Copyright (C) 2010 - DIGITEO - Allan SIMON
 * 
 * This file must be used under the terms of the CeCILL.
 * This source file is licensed as described in the file COPYING, which
 * you should have received as part of this distribution.  The terms
 * are also available at    
 * http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
 *
 */

package org.scilab.modules.metanet.graph;

import java.awt.Color;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.rmi.server.UID;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.scilab.modules.graph.ScilabGraph;
import org.scilab.modules.gui.bridge.filechooser.SwingScilabFileChooser;
import org.scilab.modules.gui.checkboxmenuitem.CheckBoxMenuItem;
import org.scilab.modules.gui.filechooser.FileChooser;
import org.scilab.modules.gui.filechooser.ScilabFileChooser;
import org.scilab.modules.gui.messagebox.ScilabModalDialog;
import org.scilab.modules.gui.messagebox.ScilabModalDialog.AnswerOption;
import org.scilab.modules.gui.messagebox.ScilabModalDialog.ButtonType;
import org.scilab.modules.gui.messagebox.ScilabModalDialog.IconType;
import org.scilab.modules.gui.tab.Tab;
import org.scilab.modules.gui.utils.SciFileFilter;
import org.scilab.modules.gui.utils.UIElementMapper;
import org.scilab.modules.gui.window.ScilabWindow;
import org.scilab.modules.metanet.Metanet;
import org.scilab.modules.metanet.MetanetTab;
import org.scilab.modules.metanet.configuration.ConfigurationManager;
import org.scilab.modules.metanet.edge.BasicEdge;
import org.scilab.modules.metanet.edge.MetanetEdgeStyle;
import org.scilab.modules.metanet.io.GraphReader;
import org.scilab.modules.metanet.io.GraphWriter;
import org.scilab.modules.metanet.io.MetanetCodec;
import org.scilab.modules.metanet.node.BasicNode;
import org.scilab.modules.metanet.node.DefaultNode;
import org.scilab.modules.metanet.utils.MetanetDialogs;
import org.scilab.modules.metanet.utils.MetanetEvent;
import org.scilab.modules.metanet.utils.MetanetFileType;
import org.scilab.modules.metanet.utils.MetanetMessages;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import com.mxgraph.io.mxCodec;
import com.mxgraph.model.mxCell;
import com.mxgraph.model.mxGeometry;
import com.mxgraph.model.mxGraphModel.mxChildChange;
import com.mxgraph.model.mxIGraphModel.mxAtomicGraphModelChange;
import com.mxgraph.util.mxConstants;
import com.mxgraph.util.mxEvent;
import com.mxgraph.util.mxEventObject;
import com.mxgraph.util.mxRectangle;
import com.mxgraph.util.mxUtils;
import com.mxgraph.view.mxStyleRegistry;


public class MetanetDiagram extends ScilabGraph {

	/**
	 * Constructor
	 */

	private Tab viewPort;
	private CheckBoxMenuItem viewPortMenu;

	private String version = "5.0.1";
	boolean directed;
	DefaultNode defaultNode;
	BasicEdge defaultEdge;




	public MetanetDiagram() {
		super();
		mxCodec codec = new mxCodec();

		try {
			// path will be added to classpath within metanet.start
			String xml = mxUtils.readFile(ClassLoader.getSystemResource("Metanet-style.xml").getPath());

			Document document = mxUtils.parse(xml);
			codec.decode(document.getDocumentElement(), getStylesheet());
		} catch (IOException e) {
			e.printStackTrace();
		}

		directed = true;
		defaultEdge = new BasicEdge(directed);
		defaultNode = new DefaultNode();

		getAsComponent().setToolTips(true);
		mxConstants.LINE_ARCSIZE = 150;
		// Forbid pending edges.
		setAllowDanglingEdges(false);


		//
		//setCloneInvalidEdges(false);
		setCloneInvalidEdges(true);

		// Override isCellEditable to filter what the user can edit
		setCellsEditable(true);
		// This enable stop editing cells when pressing Enter.
		getAsComponent().setEnterStopsCellEditing(false);
		setLabelsVisible(true);
		setHtmlLabels(true);

		setConnectableEdges(false);
		getAsComponent().setTolerance(1);

		getAsComponent().getViewport().setOpaque(false);
		getAsComponent().setBackground(Color.WHITE);



		((mxCell) getDefaultParent()).setId((new UID()).toString());
		((mxCell) getModel().getRoot()).setId((new UID()).toString());

		/* register a new style for edge */
		mxStyleRegistry.putValue("metanetEdge", new MetanetEdgeStyle());
	}

	/**
	 * Install all needed Listeners.
	 */
	public void installListeners() {

		// Property change Listener
		// Will say if a diagram has been modified or not.
		getAsComponent().addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				if (e.getPropertyName().compareTo("modified") == 0) {
					if ((Boolean) e.getOldValue() != (Boolean) e.getNewValue()) {
						updateTabTitle();
					}
				}
			}
		});

		// Add a listener to track when model is changed
		getModel().addListener(mxEvent.CHANGE, new ModelTracker());
		// Track when cells are added.
		addListener(mxEvent.CELLS_ADDED, new CellAddedTracker(this)); 
		addListener(mxEvent.REMOVE_CELLS,new CellRemovedTracker(this));
		// Track when we have to force a Block value
		addListener(MetanetEvent.FORCE_CELL_VALUE_UPDATE, new ForceCellValueUpdate());
		// Update the blocks view on undo/redo
		getUndoManager().addListener(mxEvent.UNDO, new UndoUpdateTracker());
		getUndoManager().addListener(mxEvent.REDO, new UndoUpdateTracker());

		getAsComponent().getGraphControl().addMouseListener(new MetanetMouseListener(this));

		//getAsComponent().addKeyListener(new XcosKeyListener(this));

	}

	/**
	 * CellAddedTracker
	 * Called when mxEvents.CELLS_ADDED is fired.
	 */
	private class CellAddedTracker implements mxIEventListener {
		private MetanetDiagram diagram;

		/**
		 * @param diagram diagram
		 */
		public CellAddedTracker(MetanetDiagram diagram) {
			this.diagram = diagram;
		}

		public void invoke(Object source, mxEventObject evt) {
			Object[] cells = (Object[]) evt.getProperty("cells");

			diagram.getModel().beginUpdate();
			for (int i = 0; i < cells.length; ++i) {

//				((mxCell) cells[i]).setId((new UID()).toString());

				if (cells[i] instanceof BasicNode) {
					// Update parent on cell addition
					((BasicNode) cells[i]).setParentDiagram(diagram);
				}
			}
			//fireEvent(XcosEvent.FORCE_CELL_VALUE_UPDATE, new mxEventObject(new Object[] {cells}));
			diagram.getModel().endUpdate();
		}
	}

	/**
	 * CellAddedTracker
	 * Called when mxEvents.CELLS_ADDED is fired.
	 */
	private class CellRemovedTracker implements mxIEventListener {
		private MetanetDiagram diagram;

		/**
		 * @param diagram diagram
		 */
		public CellRemovedTracker(MetanetDiagram diagram) {
			this.diagram = diagram;
		}

		public void invoke(Object source, mxEventObject evt) {
			Object[] cells = (Object[]) evt.getProperty("cells");

			diagram.getModel().beginUpdate();
			for (int i = 0; i < cells.length; ++i) {

//				((mxCell) cells[i]).setId((new UID()).toString());

				if (cells[i] instanceof BasicEdge) {
					// Update parent on cell addition
					diagram.refresh();
				}
			}
			//fireEvent(XcosEvent.FORCE_CELL_VALUE_UPDATE, new mxEventObject(new Object[] {cells}));
			diagram.getModel().endUpdate();
		}
	}


	/**
	 * ForceCellValueUpdate
	 * Called when we want a block content to update.
	 */
	private class ForceCellValueUpdate implements mxIEventListener {
		/**
		 * Constructor
		 */
		public ForceCellValueUpdate() {
			super();
		}

		public void invoke(Object source, mxEventObject evt) {
			Object[] cells = (Object[]) evt.getProperty("cells");

			getModel().beginUpdate();

			for (int i = 0; i < cells.length; ++i) {

				Object cell = cells[i];

				if (cell instanceof BasicNode) {

					//mxRectangle preferedSize = getPreferredSizeForCell(cell);
					mxGeometry cellSize = ((mxCell) cell).getGeometry();

					((mxCell) cell).setGeometry(new mxGeometry(cellSize.getX(), cellSize.getY(),
							cellSize.getWidth(),
							cellSize.getHeight()));
					cellsResized(new Object[] {cell}, new mxRectangle[]{((mxCell) cell).getGeometry()});
				}
			}
			getModel().endUpdate();
			refresh();
		}
	}

	/**
	 * modelTracker
	 * Called when mxEvents.CHANGE occurs on a model
	 */
	private class ModelTracker implements mxIEventListener {
		/**
		 * Constructor
		 */
		public ModelTracker() {
			super();
		}

		public void invoke(Object source, mxEventObject evt) {

			List<mxAtomicGraphModelChange> changes = (List<mxAtomicGraphModelChange>) (evt.getProperty("changes"));
			List<Object> objects = new ArrayList<Object>();
			getModel().beginUpdate();
			for (int i = 0; i < changes.size(); ++i) {
				if (changes.get(i) instanceof mxChildChange) {

					if (((mxChildChange) changes.get(i)).getChild() instanceof BasicNode) {
						BasicNode currentCell = (BasicNode) ((mxChildChange) changes.get(i)).getChild();
						objects.add(currentCell);
					}
				}
			}
			if (!objects.isEmpty()) {
				Object[] firedCells = new Object[objects.size()];
				for (int j = 0;  j < objects.size(); ++j) {
					firedCells[j] = objects.get(j);
				}
				//fireEvent(XcosEvent.FORCE_CELL_RESHAPE, new mxEventObject(new Object[] {firedCells}));
				fireEvent(new mxEventObject(MetanetEvent.FORCE_CELL_VALUE_UPDATE, "cells", firedCells));
			}
			getModel().endUpdate();
		}
	}

	/**
	 * MouseListener inner class
	 */
	private class MetanetMouseListener implements MouseListener {
		private MetanetDiagram diagram;

		/**
		 * @param diagram diagram
		 */
		public MetanetMouseListener(MetanetDiagram diagram) {
			this.diagram = diagram;
		}

		public void mouseClicked(MouseEvent e) {
			Object cell = getAsComponent().getCellAt(e.getX(), e.getY());

			// Double Click within empty diagram Area
			if (e.getClickCount() >= 2 && SwingUtilities.isLeftMouseButton(e) && cell == null) {
				BasicNode newNode = new BasicNode(diagram.getDefaultNode());

				newNode.getGeometry().setX((e.getX() / getView().getScale()) - newNode.getGeometry().getWidth() / 2.0);
				newNode.getGeometry().setY((e.getY() / getView().getScale()) - newNode.getGeometry().getHeight() / 2.0);
				addCell(newNode);
				return;
			}

			// Double Click within some component
			if (e.getClickCount() >= 2 && SwingUtilities.isLeftMouseButton(e) && cell != null)
			{

			}

			// Ctrl + Shift + Right Middle Click : for debug !!
			if (e.getClickCount() >= 2 && SwingUtilities.isMiddleMouseButton(e)
					&& e.isShiftDown() && e.isControlDown()) {
			}

			// Context menu
			if ((e.getClickCount() == 1 && SwingUtilities.isRightMouseButton(e))
					|| e.isPopupTrigger()
					|| MetanetMessages.isMacOsPopupTrigger(e)) {

				if (cell == null) {

				} else {
					// Display object context menu
					if (cell instanceof BasicNode) {
						BasicNode node = (BasicNode) cell;
						node.openContextMenu((ScilabGraph) getAsComponent().getGraph());
					}
					if (cell instanceof BasicEdge) {
						BasicEdge edge = (BasicEdge) cell;
						edge.openContextMenu((ScilabGraph) getAsComponent().getGraph());
					}
				}
			}
		}

		public void mouseEntered(MouseEvent e) {
		}

		public void mouseExited(MouseEvent e) {
		}

		public void mousePressed(MouseEvent e) {
		}

		public void mouseReleased(MouseEvent e) {

		}
	}

	/**
	 * Update the modified block on undo/redo
	 */
	private class UndoUpdateTracker implements mxIEventListener {
		/**
		 * Constructor
		 */
		public UndoUpdateTracker() {
			super();
		}

		public void invoke(Object source, mxEventObject evt) {

			refresh();
		}
	};

	public Object addEdge(Object edge, Object parent, Object source,
			Object target, Integer index) {

		BasicEdge newEdge = (BasicEdge)super.addEdge(new BasicEdge(defaultEdge), parent, source, target, index);



		return newEdge;

	}

    /**
     * Find the object corresponding to the given uid
     * and display a warning message.
     * 
     * @param uid - A String as UID.
     * @param message - The message to display.
     */
    public void warnObjectByUID(String uid, String message) {
	for (int i = 0; i < getModel().getChildCount(getDefaultParent()); ++i) {
	    if (getModel().getChildAt(getDefaultParent(), i) instanceof mxCell) {
		if (((mxCell) getModel().getChildAt(getDefaultParent(), i)).getId().compareTo(uid) == 0) {
			
			if (message.equals("select")) {
				addSelectionCell(getModel().getChildAt(getDefaultParent(), i));
			} else if (message.equals("unselect")) {
				removeSelectionCell(getModel().getChildAt(getDefaultParent(), i));
			}
			//setSelectionCells(new Object[]{});

		    //getAsComponent().setCellWarning(getModel().getChildAt(getDefaultParent(), i), message);
		}
	    }
	}
    }

	/**
	 * Load a file with different method depending on it extension 
	 * @param theFile File to load
	 * @param wait wait end transform 
	 */
	protected boolean transformAndLoadFile(File theFile, boolean wait) {
		final File fileToLoad = theFile;
		final MetanetFileType filetype = MetanetFileType.findFileType(fileToLoad);
		boolean result = false;

		switch (filetype) {
		case GRAPH:
			if (wait) {
				File newFile;
				newFile = filetype.exportToHdf5(fileToLoad);
				result = transformAndLoadFile(newFile, wait);
			} else {
				Thread transformAction = new Thread() {
					public void run() {
						File newFile;
						newFile = filetype.exportToHdf5(fileToLoad);
						transformAndLoadFile(newFile, false);
					}
				};
				transformAction.start();
				result = true;
			}
			break;

		case XGRAPH:
			Document document = loadMetanetDocument(theFile.getAbsolutePath());
			if (document == null) {
				MetanetDialogs.couldNotLoadFile(this);
				return false;
			}

			MetanetCodec codec = new MetanetCodec(document);

			if (getModel().getChildCount(getDefaultParent()) == 0) {
				codec.decode(document.getDocumentElement(), this);
				setModified(false);
				setSavedFile(theFile.getAbsolutePath());
				setTitle(theFile.getName().substring(0,	theFile.getName().lastIndexOf('.')));
				setChildrenParentDiagram();
			} else {
				MetanetDiagram metanetDiagram = Metanet.createANotShownDiagram();
				metanetDiagram.info(MetanetMessages.LOADING_DIAGRAM);
				codec.decode(document.getDocumentElement(), metanetDiagram);
				metanetDiagram.setModified(false);
				metanetDiagram.setSavedFile(theFile.getAbsolutePath());
				metanetDiagram.setTitle(theFile.getName().substring(0,	theFile.getName().lastIndexOf('.')));
				setChildrenParentDiagram(metanetDiagram);
				MetanetTab.showTabFromDiagram(metanetDiagram);
				metanetDiagram.info(MetanetMessages.EMPTY_INFO);
			}

			result = true;
			break;

		case HDF5:
			openDiagram(GraphReader.readDiagramFromFile(fileToLoad.getAbsolutePath()));
			generateUID();
			setModified(false);
			result = true;
			break;

		default:
			// TODO implement something more graphics 
			System.out.println("Error while opening");
		break;
		}

		return result;
	}

	/**
	 * generate unique id to all blocks in diagram
	 */
	public void generateUID() {
		for (int i = 0; i < getModel().getChildCount(getDefaultParent()); ++i) {
			if (getModel().getChildAt(getDefaultParent(), i) instanceof BasicNode) {
				BasicNode node = (BasicNode) getModel().getChildAt(getDefaultParent(), i);

				if (node.getId() == null || node.getId().compareTo("") == 0) {
					node.setId();
				}
			}
		}
	}

	public void setTitle(String title) {
		super.setTitle(title);
		updateTabTitle();
	}

	/**
	 * 
	 */
	public void updateTabTitle() {
		String tabTitle = !isModified() ? getTitle() : "* " + getTitle();
		if (getParentTab() != null) {
			getParentTab().setName(tabTitle);
			getParentTab().draw();
		}
	}

	/**
	 * @param metanetFile metanet file
	 * @return opened document
	 */
	static Document loadMetanetDocument(String metanetFile) {
		DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
		DocumentBuilder docBuilder;
		try {
			docBuilder = docBuilderFactory.newDocumentBuilder();
			return docBuilder.parse(metanetFile);
		} catch (ParserConfigurationException e) {
			return null;
		} catch (SAXException e) {
			return null;
		} catch (IOException e) {
			return null;
		}
	}

	/**
	 * Open a Diagram :
	 * If current Diagram is empty, open within it
	 * else open a new window.
	 * 
	 * @param diagramm
	 */
	public void openDiagram(HashMap<String, Object> diagramm) {
		if (diagramm != null) {
			if (getModel().getChildCount(getDefaultParent()) == 0) {
				loadDiagram(diagramm);
			} else {
				MetanetDiagram metanetDiagram = Metanet.createANotShownDiagram();
				metanetDiagram.loadDiagram(diagramm);
				setChildrenParentDiagram(metanetDiagram);
				MetanetTab.showTabFromDiagram(metanetDiagram);
			}
		} else {
			// TODO implement something more graphic
			System.out.println("Error while loading diagram");
		}
	}

	/**
	 * Read a diagram from an HDF5 file (ask for creation if the file does not exist) 
	 * @param diagramFileName file to open
	 */
	public void openDiagramFromFile(String diagramFileName) {
		if (!MetanetTab.focusOnExistingFile(diagramFileName)) {
			File theFile = new File(diagramFileName);
			info(MetanetMessages.LOADING_DIAGRAM);

			if (theFile.exists()) {
				transformAndLoadFile(theFile, false);
			} else {
				AnswerOption answer = ScilabModalDialog.show(getParentTab(), String.format(
						MetanetMessages.FILE_DOESNT_EXIST, theFile.getAbsolutePath()),
						MetanetMessages.METANET, IconType.QUESTION_ICON,
						ButtonType.YES_NO);

				if (answer == AnswerOption.YES_OPTION) {
					try {
						FileWriter writer = new FileWriter(diagramFileName);
						writer.write("");
						writer.flush();
						writer.close();
						setSavedFile(diagramFileName);
						setTitle(theFile.getName().substring(0,
								theFile.getName().lastIndexOf('.')));
					} catch (IOException ioexc) {
						JOptionPane.showMessageDialog(this.getAsComponent(), ioexc);
					}
				}
			}
			info(MetanetMessages.EMPTY_INFO);
			getUndoManager().clear();
		}
	}

	/**
	 * Update all the children of the current graph.
	 */
	public void setChildrenParentDiagram() {
		setChildrenParentDiagram(this);
	}


	/**
	 * For each Node in the argument, call its setParentDiagram method
	 * @param diagram The new parent of the nodes.
	 */
	private void setChildrenParentDiagram(MetanetDiagram diagram) {
		for (int i = 0; i < diagram.getModel().getChildCount(diagram.getDefaultParent()); i++) {
			mxCell cell = (mxCell) diagram.getModel().getChildAt(diagram.getDefaultParent(), i);
			if (cell instanceof BasicNode) {
				BasicNode node = (BasicNode) cell;
				node.setParentDiagram(diagram);

			}
		}
	}

	/**
	 * Load a Diagramm structure into current window.
	 * 
	 * @param diagramm diagram structure
	 */
	public void loadDiagram(HashMap<String, Object> diagram) {
		List<BasicNode> allNodes = (List<BasicNode>) diagram.get("Nodes");
		List<BasicEdge> allEdges = (List<BasicEdge>) diagram.get("Edges");
		DefaultNode defaultNode = (DefaultNode) diagram.get("DefaultNode");
		BasicEdge defaultEdge = (BasicEdge) diagram.get("DefaultEdge");
		boolean directed = (Boolean) diagram.get("Directed");
		HashMap<String, Object> properties = (HashMap<String, Object>) diagram.get("Properties");

		Object[] objs = new Object[allNodes.size() + allEdges.size() ];
		getModel().beginUpdate();
		for (int i = 0; i < allNodes.size(); ++i) {
			objs[i] = allNodes.get(i);
		}

		for (int i = 0; i < allEdges.size(); ++i) {
			objs[i+ allNodes.size()] = allEdges.get(i);
		}

		addCells(objs);
		getModel().endUpdate();

		//this.setTitle(fileToLoad);
		//this.getParentTab().setName(fileToLoad);
		setDirected(directed);
		setDefaultEdge(defaultEdge);
		setDefaultNode(defaultNode);

		setTitle((String) properties.get("title"));
		getParentTab().setName((String) properties.get("title"));
		// Clear all undo events in Undo Manager
		getUndoManager().clear();
		setModified(false);
		refresh();

	}

	/**
	 * @return save status
	 */
	public boolean saveDiagram() {
		boolean isSuccess = false;

		isSuccess = saveDiagramAs(getSavedFile());

		if (isSuccess) {
			setModified(false);
		}
		return isSuccess;
	}

	/**
	 * @param fileName diagram filename
	 * @return save status
	 */
	public boolean saveDiagramAs(String fileName) {

		boolean isSuccess = false;
		String writeFile = fileName;
		info(MetanetMessages.SAVING_DIAGRAM);
		if (fileName == null) {
			// Choose a filename
			SwingScilabFileChooser fc = ((SwingScilabFileChooser) ScilabFileChooser.createFileChooser().getAsSimpleFileChooser());
			fc.setTitle(MetanetMessages.SAVE_AS);
			fc.setUiDialogType(JFileChooser.SAVE_DIALOG);
			fc.setMultipleSelection(false);
			if (this.getSavedFile() != null) {
				fc.setSelectedFile(new File(this.getSavedFile()));
			}

			SciFileFilter metanetFilter = new SciFileFilter("*.xgraph", null, 0);
			SciFileFilter allFilter = new SciFileFilter("*.*", null, 1);
			fc.addChoosableFileFilter(metanetFilter);
			fc.addChoosableFileFilter(allFilter);
			fc.setFileFilter(metanetFilter);

			fc.setAcceptAllFileFilterUsed(false);
			fc.displayAndWait();

			if (fc.getSelection() == null || fc.getSelection().length == 0 || fc.getSelection()[0].equals("")) {
				info(MetanetMessages.EMPTY_INFO);
				return isSuccess;
			}
			writeFile = fc.getSelection()[0];
		}
		/* Extension checks */
		File file = new File(writeFile);
		if (!file.exists()) {
			String extension = writeFile.substring(writeFile.lastIndexOf('.') + 1);

			if (extension.equals(writeFile)) {
				/* No extension given --> .xgraph added */
				writeFile += ".xgraph";
			}
		}

		MetanetCodec codec = new MetanetCodec();
		String xml = mxUtils.getXml(codec.encode(this));

		try {
			mxUtils.writeFile(xml, writeFile);
			isSuccess = true;
		} catch (IOException e1) {
			e1.printStackTrace();
			isSuccess = false;
		}

		if (isSuccess) {
			this.setSavedFile(writeFile);
			File theFile = new File(writeFile);
			setTitle(theFile.getName().substring(0, theFile.getName().lastIndexOf('.')));
			ConfigurationManager.getInstance().addToRecentFiles(writeFile);
			ConfigurationManager.getInstance().saveConfig();
			setModified(false);
		} else {
			MetanetDialogs.couldNotSaveFile(this);
		}
		info(MetanetMessages.EMPTY_INFO);
		return isSuccess;
	}

	/**
	 * 
	 * @return
	 */

	public boolean isDirected() {
		return directed;
	}

	/**
	 * 
	 * @param directed
	 */

	public void setDirected(boolean directed) {
		this.directed = directed;
	}

	/**
	 * 
	 * @return
	 */

	public DefaultNode getDefaultNode() {
		return defaultNode;
	}

	/**
	 * 
	 * @param defaultNode
	 */

	public void setDefaultNode (DefaultNode defaultNode) {
		this.defaultNode = defaultNode;
	}

	/**
	 * 
	 * @return
	 */

	public BasicEdge getDefaultEdge() {
		return defaultEdge;
	}

	/**
	 * 
	 * @param defaultEdge
	 */

	public void setDefaultEdge (BasicEdge defaultEdge) {
		this.defaultEdge = defaultEdge;
	}

	/**
	 * Set the associated ViewPort
	 * @param viewPort the Viewport
	 */
	public void setViewPort(Tab viewPort) {
		this.viewPort = viewPort;
	}


	/**
	 * Manage the visibility of the associated viewport
	 * @param status new status
	 */
	public void setViewPortVisible(boolean status) {
		// Hide/Show parent window if the viewport is the only tab
		if (viewPort.getParentWindow().getNbDockedObjects() == 1) {
			viewPort.getParentWindow().setVisible(status);
		}
		// Hide/Show viewport tab
		viewPort.setVisible(status);

		// (Un)Check the corresponding menu
		viewPortMenu.setChecked(status);
	}

	/**
	 * Set menu used to manage Viewport visibility
	 * @param menu the menu
	 */
	public void setViewPortMenuItem(CheckBoxMenuItem menu) {
		this.viewPortMenu = menu;
	}

	/**
	 * Display the message in info bar.
	 * @param message Informations
	 */
	public void info(String message) {
		final String localMessage = message;
		if (getParentTab() != null && getParentTab().getInfoBar() != null) {
			getParentTab().getInfoBar().setText(localMessage);
		}
	}



	/**
	 * Close Metanet instance including all tabs
	 */
	public void closeDiagram() {
		closeDiagram(false);
	}

	/**
	 * Close Xcos instance including all tabs
	 * @param fromScilab true if the caller is scilab direct and not an internal call
	 */
	public void closeDiagram(boolean fromScilab) {

		boolean wantToClose = true;

		if (isModified()) {
			// The diagram has been modified
			// Ask the user want he want to do !

			AnswerOption answer; 
			if (fromScilab) {
				answer = ScilabModalDialog.show(getParentTab(), MetanetMessages.DIAGRAM_MODIFIED, MetanetMessages.METANET, 
						IconType.QUESTION_ICON, ButtonType.YES_NO);
			} else {
				answer = ScilabModalDialog.show(getParentTab(), MetanetMessages.DIAGRAM_MODIFIED, MetanetMessages.METANET, 
						IconType.QUESTION_ICON, ButtonType.YES_NO_CANCEL);
			}

			switch (answer) {
			case YES_OPTION :
				// Save the diagram
				if (!saveDiagram()) {
					//if save is canceled, cancel close windows
					wantToClose = false;
				}
				break;
			case NO_OPTION :
				break;
			case CANCEL_OPTION :
				// The user cancels
				wantToClose = false;
				break;
			default:
				break;
			}
		}


		if (wantToClose) {
			if (getParentTab() != null) {
				ScilabWindow xcosWindow = (ScilabWindow) UIElementMapper.getCorrespondingUIElement(getParentTab().getParentWindowId());
				xcosWindow.removeTab(getParentTab());
				viewPort.close();
			}
			MetanetTab.closeDiagram(this);
			setOpened(false);
		}
	}

	/**
	 * @return current version
	 */
	public String getVersion() {
		return version;
	}



	/**
	 * Returns the tooltip to be used for the given cell.
	 * @param cell block
	 * @return cell tooltip
	 */
	public String getToolTipForCell(Object cell) {
		if (cell instanceof BasicNode) {
			return ((BasicNode) cell).getToolTipText();
		} else if (cell instanceof BasicEdge) {
			return ((BasicEdge) cell).getToolTipText();
		}
		return "";
	}

	/**
	 * @param fileName HDF5 filename
	 */
	public void dumpToHdf5File(String fileName) {
		String writeFile = fileName;
		if (fileName == null) {
			FileChooser fc = ScilabFileChooser.createFileChooser();
			fc.setInitialDirectory(getSavedFile());
			fc.setMultipleSelection(false);
			fc.displayAndWait();

			if (fc.getSelection() == null || fc.getSelection().length == 0 || fc.getSelection()[0].equals("")) {
				return;
			}
			writeFile = fc.getSelection()[0];
		}

		GraphWriter.writeDiagramToFile(writeFile, this);
	}

}
